==> btanks_0.7.5800.orig.tar.gz: btanks-0.7.5800/objects/barrier.cpp <==
/* Battle Tanks Game
 * Copyright (C) 2006-2007 Battle Tanks team
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "object.h"
#include "registrar.h"
#include "config.h"
#include "alarm.h"

class Barrier : public Object {
public:
	Barrier() : Object("barrier"), _toggle(true) { pierceable = true; }
	virtual Object * clone() const;
	virtual void onSpawn();
	virtual void calculate(const float dt);
	virtual void tick(const float dt);

	virtual void serialize(mrt::Serializator &s) const {
		Object::serialize(s);
		s.add(_toggle);
	}

	virtual void deserialize(const mrt::Serializator &s) {
		Object::deserialize(s);
		s.get(_toggle);
	}

private: 
	Alarm _toggle;
};

void Barrier::onSpawn() {
	GET_CONFIG_VALUE("objects.barrier.toggle-interval", float, ti, 3.0f);
	_toggle.set(ti);
	play("closed", true);
}

void Barrier::calculate(const float dt) {
	if (_toggle.tick(dt))
		_state.fire = !_state.fire;
}

void Barrier::tick(const float dt) {
	Object::tick(dt);
	if (_state.fire && getState() == "closed") {
		cancelAll();
		play("opening", false);
		play("opened", true);
	}

	if (!_state.fire && getState() == "opened") {
		cancelAll();
		play("closing", false);
		play("closed", true);
	}
}

Object* Barrier::clone() const  {
	return new Barrier(*this);
}

REGISTER_OBJECT("barrier", Barrier, ());

==> btanks_0.8.7686.orig.tar.gz: btanks-0.8.7686/objects/barrier.cpp <==
/* Battle Tanks Game
 * Copyright (C) 2006-2008 Battle Tanks team
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/* 
 * Additional rights can be granted beyond the GNU General Public License 
 * on the terms provided in the Exception. If you modify this file, 
 * you may extend this exception to your version of the file, 
 * but you are not obligated to do so. If you do not wish to provide this
 * exception without modification, you must delete this exception statement
 * from your version and license this file solely under the GPL without exception. 
*/

#include "object.h"
#include "registrar.h"
#include "config.h"
#include "alarm.h"

class Barrier : public Object {
public:
	Barrier() : Object("barrier"), _toggle(true) { pierceable = true; }
	virtual Object * clone() const;
	virtual void on_spawn();
	virtual void calculate(const float dt);
	virtual void tick(const float dt);

	virtual void serialize(mrt::Serializator &s) const {
		Object::serialize(s);
		s.add(_toggle);
	}

	virtual void deserialize(const mrt::Serializator &s) {
		Object::deserialize(s);
		s.get(_toggle);
	}

private: 
	Alarm _toggle;
};

void Barrier::on_spawn() {
	GET_CONFIG_VALUE("objects.barrier.toggle-interval", float, ti, 3.0f);
	_toggle.set(ti);
	play("closed", true);
}

void Barrier::calculate(const float dt) {
	if (_toggle.tick(dt))
		_state.fire = !_state.fire;
}

void Barrier::tick(const float dt) {
	Object::tick(dt);
	if (_state.fire && get_state() == "closed") {
		cancel_all();
		play("opening", false);
		play("opened", true);
	}

	if (!_state.fire && get_state() == "opened") {
		cancel_all();
		play("closing", false);
		play("closed", true);
	}
}

Object* Barrier::clone() const  {
	return new Barrier(*this);
}

REGISTER_OBJECT("barrier", Barrier, ());
